<?php

namespace App\Forms;

use Nette,
	Nette\Application\UI\Form,
	App\Model\DealerManager,
	App\Presenters\DealerPresenter;


class DealerFormFactory extends Nette\Object
{
	/** @var DealerManager */
	private $dealerManager;			
	
	public function __construct(DealerManager $dealerManager)
	{
		$this->dealerManager = $dealerManager;
	}


	/**
	 * @return Form
	 */
	 
	public function create()
	{
		$form = new Form;
		
		$form->addText('dealer_name', "Dealer's\xc2\xa0name:")
			->setAttribute('placeholder', "Dealer's name")
			->setRequired("Please enter dealer's name.");

		$form->addCheckbox('infinite_credit', 'Infinite credit');
			
		$form->addText('email', 'E-mail:')
			->setType('email')
			->setAttribute('placeholder', 'E-mail')
			->setRequired("Please enter dealer's email")
			->addCondition(Form::FILLED)
				->addRule(Form::EMAIL, 'E-mail address is not valid.');
				
		$form->addText('city', 'City/Town:')
			->setAttribute('placeholder', 'City or Town');
			
		$form->addText('street', 'Street:')
			->setAttribute('placeholder', 'Street name');	
			
		$form->addText('house_num', 'House num.:')
			->setAttribute('placeholder', 'House number');
			
		$form->addHidden('id');

		$form->addCheckbox('is_active', 'Is active?');
		
		// UPDATE		
		if (isset($_GET['edit']) || !empty($_POST['id'])){
			if(isset($_GET['edit'])){
				$form['id']->setValue($_GET['edit']);
			
				$dealer = $this->dealerManager->getData($_GET['edit']);
				$address = $this->dealerManager->getAddress($_GET['edit']);
				$form->setDefaults(array(
					'dealer_name' => $dealer['dealers_name'],
					'infinite_credit' => $dealer['dealers_infinite_credit'],
					'email' => $dealer['dealers_email'],
					'is_active' => $dealer['dealers_is_active'],
					'city' => $address['address_city'],
					'street' => $address['address_street'],
					'house_num' => $address['address_house_num']
				));
			}
			
			$form->addSubmit('send', 'Update dealer');
		} else {
			$form->addSubmit('send', 'Create dealer');
		} 
		
		$form->onSuccess[] = array($this, 'formSucceeded');
		return $form;

	}


	public function formSucceeded($form, $values)
	{
		try {
			if(!empty($values->id)){
				$this->dealerManager->edit($values, $values->id);
			} else {
				$this->dealerManager->add($values);
			}
		} catch (\Exception $e) {
			$form->addError("Something went wrong. Please, try it again.");
		}
	}

}
