<?php

namespace App\Forms;

use Nette,
	Nette\Application\UI\Form,
	Nette\Security\User,
	App\Model\EventManager,
	Nette\Http\Session,
	Nette\Http\SessionSection;


class SaleFormFactory extends Nette\Object
{
	/** @var EventManager */
	private $eventManager;		
	
	/** @var Nette\Http\Session */
    private $session;

    /** @var Nette\Http\SessionSection */
    private $sessionSection;
	
	/** @var Nette\Http\Request */
    private $httpRequest;	
	
	
	public function __construct(EventManager $eventManager, Session $session, Nette\Http\Request $httpRequest)
	{
		$this->eventManager = $eventManager;
		$this->session = $session;
		$this->httpRequest = $httpRequest;
	}


	/**
	 * @return Form
	 */
	public function create()
	{
		$form = new Form;
		
		$form->addButton('reset', 'Reset');

		$form->addButton('reserve', 'Reserve Tickets');

		$form->addText('code')
			->setAttribute('placeholder', 'Reservation code');

		$form->addButton('check', 'Check Reservation');
		
		$form->addButton('submit_fake', 'Buy & Print');

		$form->addSubmit('submit', 'Buy & Print')
			->setAttribute('class', 'ajax');
		
		$form->addHidden('json');
		
		$form->onSuccess[] = array($this, 'formSucceeded');
		return $form;
	}


	public function formSucceeded($form, $values)
	{
		$this->eventManager->buyTickets($values->json, $this->httpRequest->getQuery('event_id'));
		$this->sessionSection = $this->session->getSection('ticket');
		$this->sessionSection->json = $values->json;
		$this->sessionSection->event_id = $this->httpRequest->getQuery('event_id');
	}

}
