<?php

namespace App\Forms;

use Nette,
	Nette\Application\UI\Form,
	App\Model\LayoutManager;


class TemplateFormFactory extends Nette\Object
{
	/** @var LayoutManager */
	private $layoutManager;			
	
	public function __construct(LayoutManager $layoutManager)
	{
		$this->layoutManager = $layoutManager;
	}


	/**
	 * @return Form
	 */
	public function create()
	{
		$form = new Form;
		
		$form->addText('name')
			->setAttribute('placeholder', "Template's name")
			->setRequired('You have to enter template name.');
			
		$form->addAceEditor('code')
			->setRequired('You have to enter some code.');
			
		$form->addHidden('id');
			
		// UPDATE		
		if (isset($_GET['edit']) || !empty($_POST['id'])){
			if(isset($_GET['edit'])){
				$form['id']->setValue($_GET['edit']);
			
				$data = $this->layoutManager->getTemplate($_GET['edit']);
				$form->setDefaults(array(
					'name' => $data['tickets_template_name'],
					'code' => $data['tickets_template_code']
				));
			}
			
			$form->addSubmit('send', 'Update template');
		} else {
			$form->addSubmit('send', 'Create template');
		} 
		
		$form->onSuccess[] = array($this, 'formSucceeded');
		return $form;
	}


	public function formSucceeded($form, $values)
	{
		try {			
			if(!empty($values->id)){
				$this->layoutManager->editTemplate($values, $values->id);
			} else {
				$this->layoutManager->addTemplate($values);
			}
		} catch (\Exception $e) {
			$form->addError("Something went wrong. Please, try it again.");	
		}
	}

}
