<?php

namespace App\Forms;

use Nette,
	Nette\Application\UI\Form,
	Nette\Security\User,
	App\Model\UserManager,
	App\Presenters\UserPresenter;


class UserFormFactory extends Nette\Object
{
	/** @var User 
	 *  @var UserManager 
	 */
	private $user;
	public $userManager;			
	
	public function __construct(User $user, UserManager $userManager)
	{
		$this->user = $user;
		$this->userManager = $userManager;
	}


	/**
	 * @return Form
	 */
	 
	public function create()
	{
		$roles = $this->userManager->getRoles();
		$dealers = $this->userManager->getDealers();
				
		$form = new Form;
		
		$form->addText('username', 'Username:')
			->setAttribute('placeholder', 'Username')
			->setRequired("Please enter user's username.");
			
		$form->addTextarea('description', "User\xc2\xa0description:")
			->setAttribute('placeholder', 'User description');
			
		$form->addSelect('role', 'User role:')
			->setItems($roles);

		$form->addPassword('password', 'Password:')
			->setAttribute('placeholder', 'Password');
			
		$form->addText('email', 'E-mail:')
			->setType('email')
			->setAttribute('placeholder', 'E-mail')
			->setRequired("Please enter user's email")
			->addCondition(Form::FILLED)
				->addRule(Form::EMAIL, 'E-mail address is not valid.');
			
		$form->addSelect('dealer', 'Dealer:')
			->setItems($dealers)
			->setPrompt("--- User's dealer ---")
			->setRequired("Please select user's dealer.");
			
		$form->addText('city', 'City/Town:')
			->setAttribute('placeholder', 'City or Town');
			
		$form->addText('street', 'Street:')
			->setAttribute('placeholder', 'Street name');	
			
		$form->addText('house_num', 'House num.:')
			->setAttribute('placeholder', 'House number');	
			
		$form->addHidden('id');

		$form->addCheckbox('is_active', 'Is active?');
		
		// UPDATE		
		if (isset($_GET['edit']) || !empty($_POST['id'])){
			if(isset($_GET['edit'])){
				$form['id']->setValue($_GET['edit']);
			
				$user = $this->userManager->getData($_GET['edit']);
				$address = $this->userManager->getAddress($_GET['edit']);
				$form->setDefaults(array(
					'username' => $user['users_name'],
					'description' => $user['users_description'],
					'role' => $user['users_role'],
					'email' => $user['users_email'],
					'dealer' => $user['users_dealer'],
					'is_active' => $user['users_is_active'],
					'city' => $address['address_city'],
					'street' => $address['address_street'],
					'house_num' => $address['address_house_num']
				));
			}
			
			$form->addSubmit('send', 'Update user');
		} else {
			$form['password']->setRequired("Please enter user's password.");
			$form->addSubmit('send', 'Create user');
		} 
		
		$form->onSuccess[] = array($this, 'formSucceeded');
		return $form;

	}


	public function formSucceeded($form, $values)
	{
		try {
			if(!empty($values->id)){
				$this->userManager->edit($values, $values->id);
			} else {
				$this->userManager->add($values);
			}
		} catch (\Exception $e) {
			$form->addError("Something went wrong. Please, try it again.");
		}
	}

}
