<?php

namespace App\Model;

use Nette,
	Nette\Utils\Strings,
	Nette\Utils\ArrayHash;


/**
 * Dealers management.
 */
class DealerManager extends Nette\Object
{
	const
		DEALERS_TABLE_NAME = 'dealers',
		DEALERS_ID = 'id_dealers',
		DEALERS_NAME = 'dealers_name',
		DEALERS_IS_ACTIVE = 'dealers_is_active',
		DEALERS_CREDIT = 'dealers_credit',
		DEALERS_INFINITE_CREDIT = 'dealers_infinite_credit',
		DEALERS_EMAIL = 'dealers_email',
		DEALERS_ADDRESS = 'dealers_address_id',
        DEALERS_IS_REMOVED = 'dealers_is_active';


	/** @var Nette\Database\Context */
	private $database;


	public function __construct(Nette\Database\Context $database)
	{
		$this->database = $database;
	}


	/**
	 * Adds new dealer.
	 * @param  array
	 * @return void
	 * @throws DuplicateNameException
	 */
	public function add($values)
	{
		try {
			if ($values->city != "" || $values->street != "" || $values->house_num != "") {
				$a = $this->database->table('address')->insert(array(
					'address_city' => $values->city,
					'address_street' => $values->street,
					'address_house_num' => $values->house_num
				));
				$address_id = $a->id_address;
			} else {
				$address_id = 1;
			}
			
			$this->database->table(self::DEALERS_TABLE_NAME)->insert(array(
				self::DEALERS_NAME => $values->dealer_name,
				self::DEALERS_IS_ACTIVE => $values->is_active,
				self::DEALERS_INFINITE_CREDIT => $values->infinite_credit,
				self::DEALERS_EMAIL => $values->email,
				self::DEALERS_ADDRESS => $address_id
			));	
		} catch (Nette\Database\UniqueConstraintViolationException $e) {
			throw new \Exception('The name '.$values->dealer_name.' already exists.');		
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	/**
	 * Updates dealer.
	 * @param  array
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function edit($values, $id)
	{
		try {
			$d = $this->database->table(self::DEALERS_TABLE_NAME)->get($id);

			if ($values['city'] != "" || $values['street'] != "" || $values['house_num'] != "") {
				$this->database->table('address')->where('id_address', $d->dealers_address_id)->update(array(
					'address_city' => $values['city'],
					'address_street' => $values['street'],
					'address_house_num' => $values['house_num']
				));
				$address_id = $d->dealers_address_id;
			} else {
				$address_id = 1;
			}

			$this->database->table(self::DEALERS_TABLE_NAME)->get($id)->update(array(				 
				self::DEALERS_NAME => $values['dealer_name'],
				self::DEALERS_IS_ACTIVE => $values['is_active'],
				self::DEALERS_INFINITE_CREDIT => $values['infinite_credit'],
				self::DEALERS_EMAIL => $values['email'],
				self::DEALERS_ADDRESS => $address_id
			));
			
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	/**
	 * Deletes dealer.
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function delete($id)
	{
		try {
            $this->database->table('users')->where('users_delaers_id', $id)->update(array(
                'users_is_active' => "0"
            ));

			$this->database->table(self::DEALERS_TABLE_NAME)->get($id)->update(array(
				self::DEALERS_IS_ACTIVE => "0",
                self::DEALERS_IS_REMOVED => "1"
			));

		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}

	
	/**
	 * Selects dealer's address
	 */
	public function getAddress($id)
	{
		$dealer = $this->database->table(self::DEALERS_TABLE_NAME)->where(self::DEALERS_ID, $id)->select(self::DEALERS_ADDRESS)->fetch();
		$address = $this->database->table('address')->get($dealer->dealers_address);
		
		return $address;
	}
	
	
	/**
	 * Selects all dealer's information for update.
	 * @param array
	 * @return array $dealer_data
	 */
	public function getData($id)
	{			
		$dealer_data = $this->database->table(self::DEALERS_TABLE_NAME)->where(self::DEALERS_ID, $id)->fetch();	
		
		return $dealer_data; 
	}
	
	
	/**
	 * Adds credit to dealer's.
	 * @param int
	 * @param int
	 */
	public function addCredit($id, $val)
	{			
		$this->database->table(self::DEALERS_TABLE_NAME)->where(self::DEALERS_ID, $id)->update(array(
			self::DEALERS_CREDIT => new \Nette\Database\SqlLiteral(self::DEALERS_CREDIT."+".$val)
		));	

		$this->creditRefresh($id);
	}
	
	
	/**
	 * Deducts dealer's credit.
	 * @param int
	 * @param int
	 */
	public function deductCredit($id, $val)
	{			
		$this->database->table(self::DEALERS_TABLE_NAME)->where(self::DEALERS_ID, $id)->update(array(
			self::DEALERS_CREDIT => new \Nette\Database\SqlLiteral(self::DEALERS_CREDIT."-".$val)
		));	

		$this->creditRefresh($id);
	}

	public function creditRefresh($id)
	{
		$credit = $this->database->table(self::DEALERS_TABLE_NAME)->where(self::DEALERS_ID, $id)->select(self::DEALERS_CREDIT)->fetch();
		Nette\Environment::getUser()->getIdentity()->users_credit = $credit->dealers_credit;
	}
	
}