<?php

namespace App\Model;

use Nette,
	Nette\Utils\Strings,
	Nette\Utils\ArrayHash,
	Rodrigues\Barcode\Barcode,
	Endroid\QrCode\QrCode;
	


/**
 * Events management.
 */
class EventManager extends Nette\Object
{
	const
		EVENTS_TABLE_NAME = 'events',
		EVENTS_ID = 'id_events',
		EVENTS_NAME = 'events_name',
		EVENTS_DESCRIPTION = 'events_description',
		EVENTS_KIND = 'events_kind_id',
		EVENTS_DATE = 'events_date',
		EVENTS_TIME = 'events_time',
		EVENTS_ADDRESS = 'events_address_id',
		EVENTS_IS_ACTIVE = 'events_is_active',
		EVENTS_TICKETS_TEMPLATE_ID = 'events_tickets_template_id',
		EVENTS_SEATS_LAYOUT_ID = 'events_seats_layout_id',
		EVENTS_SEATS_LAYOUT_EVENTS_MIN = 'events_seats_layout_events_min',
		EVENTS_SEATS_LAYOUT_EVENTS_MAX = 'events_seats_layout_events_max',
		
		KINDS_TABLE_NAME = 'kinds',
		KINDS_ID = 'id_kinds',
		KINDS_NAME = 'kinds_name',
				
		SEATS_LAYOUT_TABLE_NAME = 'seats_layout',
		SEATS_LAYOUT_ID = 'id_seats_layout',
		SEATS_LAYOUT_NAME = 'seats_layout_name',
		SEATS_LAYOUT_FILENAME = 'seats_layout_filename',
		SEATS_LAYOUT_IS_ACTIVE = 'seats_layout_is_active',
		
		SEATS_TABLE_NAME = 'seats',
		SEATS_ID = 'id_seats',
		SEATS_SEATS_LAYOUT_ID = 'seats_seats_layout_id',
		SEATS_NAME = 'seats_name',
		
		ZONES_SEATS_TABLE_NAME = 'zones_seats',
		ZONES_SEATS_SEATS_ID = 'zones_seats_seats_id',
		ZONES_SEATS_ZONES_ID = 'zones_seats_zones_id',
		
		ZONES_TABLE_NAME = 'zones',
		ZONES_ID = 'id_zones',
		ZONES_PRICE = 'zones_price',
		
		TICKETS_TABLE_NAME = 'tickets',
		TICKETS_ID = 'id_tickets',
		TICKETS_SEATS_ID = 'tickets_seats_id',
		TICKETS_EVENTS_ID = 'tickets_events_id',
		TICKETS_CODE = 'tickets_code',
		TICKETS_SALES_TIME = 'tickets_sales_time',
		TICKETS_USED_TIME = 'tickets_used_time',
		TICKETS_USERS_ID = 'tickets_users_id',
		TICKETS_PRICE = 'tickets_price',
		TICKETS_STATE = 'tickets_state',
		
		TICKETS_TEMPLATE_TABLE_NAME = 'tickets_template',
		TICKETS_TEMPLATE_ID = 'id_tickets_template',
		TICKETS_TEMPLATE_NAME = 'tickets_template_name',
		TICKETS_TEMPLATE_CODE = 'tickets_template_code',
		TICKETS_TEMPLATE_IS_ACTIVE = 'tickets_template_is_active',
		TICEKTS_RESERVATIONS_ID = 'tickets_reservations_id',
		
		SEATS_LAYOUT_EVENTS_TABLE_NAME = 'seats_layout_events',
		SEATS_LAYOUT_EVENTS_ID = 'id_seats_layout_events',
		SEATS_LAYOUT_EVENTS_AVAILABLE = 'seats_layout_events_available',
		SEATS_LAYOUT_EVENTS_SEATS_ID= 'seats_layout_events_seats_id',
		SEATS_LAYOUT_EVENTS_EVENTS_ID = 'seats_layout_events_events_id',
		SEATS_LAYOUT_EVENTS_STATUS = 'seats_layout_events_status';


	/** @var Nette\Database\Context */
	private $database;
	
	/** @var Nette\Security\User */
	private $user;


	public function __construct(Nette\Database\Context $database, Nette\Security\User $user)
	{
		$this->database = $database;
		$this->user = $user;
	}


	/**
	 * Adds new event.
	 * @param  array
	 * @return void
	 * @throws DuplicateNameException
	 */
	public function add($values)
	{
		try {
			if (empty($values->template)){
				$values->template = NULL;
			}
			
			if (empty($values->layout)){
				$values->layout = NULL;
			}
			
			if ($values->city != "" || $values->street != "" || $values->house_num != "") {
				$a = $this->database->table('address')->insert(array(
					'address_city' => $values->city,
					'address_street' => $values->street,
					'address_house_num' => $values->house_num
				));
				$address_id = $a->id_address;
			} else {
				$address_id = 1;
			}

			$k = $this->database->table(self::KINDS_TABLE_NAME)->where(self::KINDS_NAME, $values->kind)->select(self::KINDS_ID)->fetch();
			if (empty($k) || $k == "0") {
				$k = $this->database->table(self::KINDS_TABLE_NAME)->insert(array(
					     self::KINDS_NAME => $values->kind
					 ));
			}
			
			$event = $this->database->table(self::EVENTS_TABLE_NAME)->insert(array(
				self::EVENTS_NAME => $values->event_name,
				self::EVENTS_DESCRIPTION => $values->description,
				self::EVENTS_KIND => $k->id_kinds,
				self::EVENTS_DATE => $values->date,
				self::EVENTS_TIME => $values->time,
				self::EVENTS_TICKETS_TEMPLATE_ID => $values->template,
				self::EVENTS_SEATS_LAYOUT_ID => $values->layout,
				self::EVENTS_ADDRESS => $address_id
			));	
			
			if ($values->layout != NULL) {
				$seats = $this->database->table(self::SEATS_TABLE_NAME)->where(self::SEATS_SEATS_LAYOUT_ID, $values->layout)
						 ->select(self::SEATS_ID)->fetchAll();

				$seats_ids = [];

				foreach ($seats as $key => $seats_id) {
					array_push($seats_ids, ["seats_layout_events_seats_id" => $seats_id]);
				}

				$this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->insert($seats_ids);

				$max = $this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->max(self::SEATS_LAYOUT_EVENTS_ID);
				$min = $max - count($seats) + 1;
				
				$this->database->table(self::EVENTS_TABLE_NAME)->get($event->id_events)->update(array(
					self::EVENTS_SEATS_LAYOUT_EVENTS_MIN => $min,
					self::EVENTS_SEATS_LAYOUT_EVENTS_MAX => $max
				));
			}

			return $event->id_events;
			
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");
		}
	}
	
	
	/**
	 * Updates event.
	 * @param  array
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function edit($values, $id)
	{
		try {
			if (empty($values['template'])){
				$values['template'] = NULL;
			}
			
			if (empty($values['layout'])){
				$values['layout'] = NULL;
			}
			
			$event = $this->database->table(self::EVENTS_TABLE_NAME)->get($id);

			if ($values['city'] != "" || $values['street'] != "" || $values['house_num'] != "") {
				$this->database->table('address')->where('id_address', $event->events_address_id)->update(array(
					'address_city' => $values['city'],
					'address_street' => $values['street'],
					'address_house_num' => $values['house_num']
				));
				$address_id = $event->events_address_id;
			} else {
				$address_id = 1;
			}

			$k = $this->database->table(self::KINDS_TABLE_NAME)->where(self::KINDS_NAME, $values['kind'])->fetch();
			if (empty($k) || $k == "0") {
				$k = $this->database->table(self::KINDS_TABLE_NAME)->insert(array(
							self::KINDS_NAME => $values['kind']
						));
			}

			$this->database->table(self::EVENTS_TABLE_NAME)->get($id)->update(array(				 
				self::EVENTS_NAME => $values['event_name'], 
				self::EVENTS_DESCRIPTION => $values['description'],
				self::EVENTS_KIND => $k->id_kinds,
				self::EVENTS_DATE => $values['date'],
				self::EVENTS_TIME => $values['time'],
				self::EVENTS_TICKETS_TEMPLATE_ID => $values['template'],
				self::EVENTS_SEATS_LAYOUT_ID => $values['layout'],
				self::EVENTS_ADDRESS => $address_id
			));
			
			if ($values['layout'] != NULL && $event->events_seats_layout_id != $values['layout']) {
				$this->refactorSeats($values['layout'], $id);
			}
			
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please try it again.");	
		}
	}


	/**
	 * Refactors events seats.
	 * @param int
	 * @param int
	 */
	public function refactorSeats($layouts_id, $events_id)
	{
		try {
			$seats = $this->database->table(self::SEATS_TABLE_NAME)->where(self::SEATS_SEATS_LAYOUT_ID, $layouts_id)
					 ->select(self::SEATS_ID)->fetchAll();
			
			$seats_ids = [];

			foreach ($seats as $key => $seats_id) {
				array_push($seats_ids, ["seats_layout_events_seats_id" => $seats_id]);
			}

			$this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->insert($seats_ids);

			$max = $this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->max(self::SEATS_LAYOUT_EVENTS_ID);
			$min = $max - count($seats) + 1;
			
			$this->database->table(self::EVENTS_TABLE_NAME)->get($events_id)->update(array(
				self::EVENTS_SEATS_LAYOUT_EVENTS_MIN => $min,
				self::EVENTS_SEATS_LAYOUT_EVENTS_MAX => $max
			));
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	
	/**
	 * Deletes event.
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function delete($id)
	{
		try {
			$this->database->table(self::EVENTS_TABLE_NAME)->get($id)->update(array(
				self::EVENTS_IS_ACTIVE => "0"
			));	
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	
	/**
	 * Selects event's address
	 */
	public function getAddress($id)
	{
		$event = $this->database->table(self::EVENTS_TABLE_NAME)->where(self::EVENTS_ID, $id)->select(self::EVENTS_ADDRESS)->fetch();
		$address = $this->database->table('address')->get($event->events_address);
		
		return $address;
	}
	
	
	/**
	 * Selects all layouts for select input in EventForm.
	 * @return array $layouts
	 */
	public function getLayouts()
	{
		$layouts = $this->database->table(self::SEATS_LAYOUT_TABLE_NAME)->where(self::SEATS_LAYOUT_IS_ACTIVE, 1)->select(self::SEATS_LAYOUT_NAME.','.self::SEATS_LAYOUT_ID)
				   ->fetchPairs(self::SEATS_LAYOUT_ID, self::SEATS_LAYOUT_NAME);		
		
		return $layouts;
	}
	
	
	/**
	 * Selects all templates for select input in EventForm.
	 * @return array $templates
	 */
	public function getTemplates()
	{
		$templates = $this->database->table(self::TICKETS_TEMPLATE_TABLE_NAME)->where(self::TICKETS_TEMPLATE_IS_ACTIVE, 1)->select(self::TICKETS_TEMPLATE_NAME.','.self::TICKETS_TEMPLATE_ID)
				   ->fetchPairs(self::TICKETS_TEMPLATE_ID, self::TICKETS_TEMPLATE_NAME);		
		
		return $templates;
	}
	
	
	/**
	 * Selects all event's information for update.
	 * @param string
	 * @return array $event_data
	 */
	public function getData($id)
	{			
		$event_data = $this->database->table(self::EVENTS_TABLE_NAME)->where(self::EVENTS_ID, $id)->fetch();
		
		return $event_data; 
	}
	
	
	/**
	 * Selects event information for sale.
	 * @param string
	 * @return array $event_data
	 */
	public function getSale($event_id)
	{			
		$layout_data = $this->database->table(self::EVENTS_TABLE_NAME)->where(self::EVENTS_ID, $event_id)
			->select('seats_layout.'.self::SEATS_LAYOUT_ID.',seats_layout.'.self::SEATS_LAYOUT_NAME.',seats_layout.'.self::SEATS_LAYOUT_FILENAME.','.self::EVENTS_SEATS_LAYOUT_EVENTS_MIN.','.self::EVENTS_SEATS_LAYOUT_EVENTS_MAX)->fetch();
	
		$zones_data = $this->database->query("
			SELECT zones_name, zones_coords, id_zones, zones_price
			FROM zones
			JOIN zones_seats ON zones_seats_zones_id = id_zones
			JOIN seats ON seats_seats_layout_id = ?
			GROUP BY id_zones
		", $layout_data->id_seats_layout)->fetchAll();
		
		foreach($zones_data as $zone){
			$seats_data[$zone['id_zones']] = $this->database->query("
				SELECT seats_layout_events.seats_layout_events_available, seats.seats_name, seats.seats_price/100*? AS seats_price, seats.id_seats, zones.zones_row_seats_num 
				FROM zones_seats 
				JOIN seats ON zones_seats.zones_seats_seats_id = seats.id_seats
				JOIN zones ON zones_seats.zones_seats_zones_id = zones.id_zones 
				JOIN seats_layout_events ON seats.id_seats = seats_layout_events.seats_layout_events_seats_id
				WHERE (seats_seats_layout_id = ?) AND (zones_seats_zones_id = ?) AND (id_seats_layout_events >= ?) AND (id_seats_layout_events <= ?)
			", $zone['zones_price'], $layout_data->id_seats_layout, $zone['id_zones'], $layout_data->events_seats_layout_events_min, $layout_data->events_seats_layout_events_max)->fetchAll();				
		}
		
		$event = $this->database->table(self::EVENTS_TABLE_NAME)->where(self::EVENTS_ID, $_GET['event_id'])->select(self::EVENTS_NAME)->fetch();
		
		$event_data['layout'] = $layout_data;
		$event_data['zones'] = $zones_data;
		$event_data['seats'] = $seats_data;
		$event_data['event'] = $event->events_name;
		
		return $event_data; 
	}
	
	
	/**
	 * Blocks seats.
	 * @param string
	 */
	public function ticketBlock($seats_id, $events_id)
	{		
		try {
			$e = $this->database->table(self::EVENTS_TABLE_NAME)->where(self::EVENTS_ID, $events_id)->select(self::EVENTS_SEATS_LAYOUT_EVENTS_MAX.','.self::EVENTS_SEATS_LAYOUT_EVENTS_MIN)->fetch();
			$s = $this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->where(array(
					self::SEATS_LAYOUT_EVENTS_SEATS_ID => $seats_id,
					self::SEATS_LAYOUT_EVENTS_STATUS => 0,
					self::SEATS_LAYOUT_EVENTS_ID.">= ?" => $e->events_seats_layout_events_min,
					self::SEATS_LAYOUT_EVENTS_ID."<= ?" => $e->events_seats_layout_events_max
				 ))->update(array(
					self::SEATS_LAYOUT_EVENTS_STATUS => 1
				 ));	
				 
			if (empty($s) || $s == "0") {
				header($_SERVER['SERVER_PROTOCOL']. ' 500 Duplicate Seats Selection', true, 500);
				die(json_encode(array('message' => 'The selected seat is already occupied. Please select a different one.')));
			}
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}	


	/**
	 * Makes seats unavailable and generates unique code for reservation.
	 * @param string
	 * @return string
	 */
	function reserveTickets($jsonStr, $events_id)
	{
		$json = json_decode($jsonStr, TRUE);	
		
		try {
			do {
				$random = Strings::random(5);
				$code = $this->database->table('reservations')->where('reservations_code', $random)->select('id_reservations')->fetch();
			} while (!empty($code));

			$this->database->table('reservations')->insert(array(
				'reservations_code' => $random,
				'reservations_time' => date('Y-m-d h:i:s', time()),
				'reservations_users_id' => $this->user->getId(),
				'reservations_events_id' => $events_id
			));	

			$reservation = $this->database->table('reservations')->where('reservations_code', $random)->select('id_reservations')->fetch();
			$e = $this->database->table(self::EVENTS_TABLE_NAME)->get($events_id);

			foreach($json as $key => $seat) {
				$s = $this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->where(array(
					self::SEATS_LAYOUT_EVENTS_SEATS_ID => $seat['seats_id'],
					self::SEATS_LAYOUT_EVENTS_ID.">= ?" => $e->events_seats_layout_events_min,
					self::SEATS_LAYOUT_EVENTS_ID."<= ?" => $e->events_seats_layout_events_max
				))->fetch();
				if ($s->seats_layout_events_available == "VIP_1") {
					$s->update(array(
						self::SEATS_LAYOUT_EVENTS_AVAILABLE => "VIP_0_res"
					));	
				} elseif ($s->seats_layout_events_available == "1") {
					$s->update(array(
						self::SEATS_LAYOUT_EVENTS_AVAILABLE => "0_res"
					));	
				}
				
				$this->database->table('reservations_seats')->insert(array(
					'reservations_seats_reservations_id' => $reservation->id_reservations,
					'reservations_seats_seats_id' => $seat['seats_id'],
					'reservations_seats_seats_price' => $seat['seats_price']
				));	
			}

			return $random;

		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
		
	}	
		

	/**
	 * Makes seats unavailable and generates unique code for ticket(s).
	 * @param string
	 */
	function buyTickets($jsonStr, $events_id)
	{
		$json = json_decode($jsonStr, TRUE);	
		$reservation = "";

		try {
			$e = $this->database->table(self::EVENTS_TABLE_NAME)->get($events_id);

			var_dump($json);
			foreach($json as $key => $seat) {
				$s = $this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->where(array(
					self::SEATS_LAYOUT_EVENTS_SEATS_ID => $seat['seats_id'],
					self::SEATS_LAYOUT_EVENTS_ID.">= ?" => $e->events_seats_layout_events_min,
					self::SEATS_LAYOUT_EVENTS_ID."<= ?" => $e->events_seats_layout_events_max
				))->fetch();

				do {
					$random = Strings::random();
					$code = $this->database->table(self::TICKETS_TABLE_NAME)->where(self::TICKETS_CODE, $random)->select(self::TICKETS_ID)->fetch();
				} while (!empty($code));	

				if ($s->seats_layout_events_available == "VIP_0_res" || $s->seats_layout_events_available == "0_res") {
					if ($reservation == "") {
						$reservation = $this->database->query("
							SELECT id_reservations
							FROM reservations_seats
							JOIN reservations ON id_reservations = reservations_seats_reservations_id
							JOIN seats_layout_events ON seats_layout_events_seats_id = reservations_seats_seats_id
							WHERE reservations_seats_seats_id = ? AND reservations_events_id = ? AND reservations_is_active = 1
						", $seat['seats_id'], $events_id)->fetch();
					}

					$this->database->query("
						UPDATE reservations_seats
						JOIN reservations ON id_reservations = reservations_seats_reservations_id
						JOIN seats_layout_events ON seats_layout_events_seats_id = reservations_seats_seats_id
						SET was_buyed = 1
						WHERE reservations_seats_seats_id = ? AND reservations_events_id = ? AND (id_seats_layout_events >= ?) AND (id_seats_layout_events <= ?) AND reservations_is_active = 1
					", $seat['seats_id'], $events_id, $e->events_seats_layout_events_min, $e->events_seats_layout_events_max);

					$this->database->table(self::TICKETS_TABLE_NAME)->insert(array(
						self::TICKETS_CODE => $random,
						self::TICKETS_SEATS_ID => $seat['seats_id'],
						self::TICKETS_EVENTS_ID => $events_id,
						self::TICKETS_SALES_TIME => date('Y-m-d h:i:s', time()),
						self::TICKETS_USERS_ID => $this->user->getId(),
						self::TICKETS_PRICE => $seat['seats_price'],
						self::TICEKTS_RESERVATIONS_ID => $reservation->id_reservations
					));	
				} else {
					$this->database->table(self::TICKETS_TABLE_NAME)->insert(array(
						self::TICKETS_CODE => $random,
						self::TICKETS_SEATS_ID => $seat['seats_id'],
						self::TICKETS_EVENTS_ID => $events_id,
						self::TICKETS_SALES_TIME => date('Y-m-d h:i:s', time()),
						self::TICKETS_USERS_ID => $this->user->getId(),
						self::TICKETS_PRICE => $seat['seats_price'],
						self::TICEKTS_RESERVATIONS_ID => NULL
					));	
				}

				switch($s->seats_layout_events_available) {
					case '0_res':
					case '1':
						$s->update(array(
							self::SEATS_LAYOUT_EVENTS_AVAILABLE => "0"
						));	
						break;
					case 'VIP_0_res':
					case 'VIP_1':
						$s->update(array(
							self::SEATS_LAYOUT_EVENTS_AVAILABLE => "VIP_0"
						));	
				}

			}

			if ($reservation != "") {
				$not_buyed = $this->database->query("
					SELECT seats_layout_events_seats_id
					FROM seats_layout_events
					JOIN reservations_seats ON reservations_seats_seats_id = seats_layout_events_seats_id
					JOIN reservations ON id_reservations = reservations_seats_reservations_id
					WHERE id_reservations = ? AND (id_seats_layout_events >= ?) AND (id_seats_layout_events <= ?) AND was_buyed = 0
				", $reservation->id_reservations, $e->events_seats_layout_events_min, $e->events_seats_layout_events_max)->fetchAll();

				foreach ($not_buyed as $not) {
					$this->database->query("
						UPDATE seats_layout_events
						JOIN reservations_seats ON reservations_seats_seats_id = seats_layout_events_seats_id
						JOIN reservations ON id_reservations = reservations_seats_reservations_id
						SET seats_layout_events_available = IF(seats_layout_events_available = '0_res', '1', 'VIP_1')
						WHERE seats_layout_events_seats_id = ? AND id_reservations = ? AND (id_seats_layout_events >= ?) AND (id_seats_layout_events <= ?) AND was_buyed = 0
					", $not->seats_layout_events_seats_id, $reservation->id_reservations, $e->events_seats_layout_events_min, $e->events_seats_layout_events_max);
				}

				$this->database->table('reservations')->get($reservation->id_reservations)->update(array(
					'reservations_is_active' => 0
				));
			}

		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}

	/**
	 * Checks reservations code and returns seats id's
	 */
	function checkReservation($code, $event_id)
	{
		try {
			$e = $this->database->table(self::EVENTS_TABLE_NAME)->get($event_id);

			$seats = $this->database->query('
				SELECT reservations_seats_seats_id, reservations_seats_seats_price, id_reservations, reservations_is_active, seats_layout_events_available
				FROM reservations_seats
				JOIN reservations ON id_reservations = reservations_seats_reservations_id
				JOIN seats_layout_events ON seats_layout_events_seats_id = reservations_seats_seats_id
				WHERE reservations_code = ? AND reservations_events_id = ? AND (id_seats_layout_events >= ?) AND (id_seats_layout_events <= ?)
			 ', $code, $event_id, $e->events_seats_layout_events_min, $e->events_seats_layout_events_max)->fetchAll();			

			if (empty($seats) || $seats == "0") {
				header($_SERVER['SERVER_PROTOCOL']. ' 500 Seats Not Found', true, 500);
				die(json_encode(array('message' => 'The reservation code was not found for this event.')));
			}

			$list = array();

			foreach ($seats as $seat) {	
				if ($seat->reservations_is_active == "0") {
					header($_SERVER['SERVER_PROTOCOL']. ' 500 Reservation Code Was Used', true, 500);
					die(json_encode(array('message' => 'The reservation code was used already!')));
				} 
			
				$s = $this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->where(array(
					self::SEATS_LAYOUT_EVENTS_SEATS_ID => $seat->reservations_seats_seats_id,
					self::SEATS_LAYOUT_EVENTS_ID.">= ?" => $e->events_seats_layout_events_min,
					self::SEATS_LAYOUT_EVENTS_ID."<= ?" => $e->events_seats_layout_events_max
				))->fetch();

				$list[$s->seats_layout_events_seats_id] = $s->seats_layout_events_available;
			}

			return json_encode($list);

		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	
	/**
	 * Unblocks seat so it can be in sale again.
	 */
	function ticketUnblock($seats_id, $events_id){
		$e = $this->database->table(self::EVENTS_TABLE_NAME)->where(self::EVENTS_ID, $events_id)->select(self::EVENTS_SEATS_LAYOUT_EVENTS_MAX.','.self::EVENTS_SEATS_LAYOUT_EVENTS_MIN)->fetch();		
		$this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->where(array(
			self::SEATS_LAYOUT_EVENTS_SEATS_ID => $seats_id,
			self::SEATS_LAYOUT_EVENTS_STATUS => 1,
			self::SEATS_LAYOUT_EVENTS_ID.">= ?" => $e->events_seats_layout_events_min,
			self::SEATS_LAYOUT_EVENTS_ID."<= ?" => $e->events_seats_layout_events_max
		))->update(array(
			self::SEATS_LAYOUT_EVENTS_STATUS => 0
		));						
	}


	/**
	 * Unblocks all seats after refresh.
	 */
	function ticketUnblockRefresh($jsonStr, $events_id){
		$json = json_decode($jsonStr, TRUE);

		foreach ($json as $key => $seat) {
			$this->ticketUnblock($seat['seats_id'], $events_id);
		}				
	}
	
	
	/**
	 * Control if ticket code is valid and unused.
	 * @param int
	 * @param int
	 * @param string
	 * @return string
	 */
	 function ticketValidation($id, $code){
	 	try {
	 		$params[self::TICKETS_CODE] = $code; 
			if ($id != NULL) { 
				$params[self::TICKETS_EVENTS_ID] = $id;
			}

	 		$validation = $this->database->table(self::TICKETS_TABLE_NAME)->select(self::TICKETS_STATE)->where($params)->fetch();
			
			if (isset($validation->tickets_state)){
				if ($validation->tickets_state == "used") {
					return "The ticket's code ".$code." was used already!";
				} else {
					$validation = true;
				}
			} else {
				return "The ticket's code ".$code." was not found for this event!";
			}

			$this->database->table(self::TICKETS_TABLE_NAME)->where(self::TICKETS_CODE, $code)->update(array(
				self::TICKETS_STATE => "used",
				self::TICKETS_USED_TIME => date('Y-m-d h:i:s', time())
			));
			
			return $validation;
	 	} catch (\Exception $e) {
			throw new \Exception($e);
	 	}
		
	 }
	 
	 
	 /*
	  * Returns filled template for print.latte
	  * @param string
	  * @return array
	  */
	 public function ticketPrint($jsonStr, $event_id) {
		$filled = array();	
		$search = array('{kind}', '{event}', '{date}', '{time}', '{seat}', '{bar_code}', '{qr_code}', '{unique_code}', '{price}', '{address-city}', '{address-street}', '{address-house-number}');

		try {
			$temp = $this->database->table(self::EVENTS_TABLE_NAME)->where(self::EVENTS_ID, $event_id)
					->select("tickets_template.".self::TICKETS_TEMPLATE_CODE)->fetch();
			$template = $temp->tickets_template_code;
						
			$first = strrpos($template, "<body>");
			$last = strrpos($template, "</body>");
			$head = substr($template, 0, $first)."<body>";
			$body = substr($template, $first, $last);
			$h = substr_replace($head, "@media print{*{-webkit-print-color-adjust:exact;}}", strpos($head, '</style>'), 0);
			$head = $h;
			
			// Barcode
			$barcode = new Barcode;
			if (strpos($template,'{bar_code}') !== false) {
				$h = substr_replace($head, $barcode->getCssStyle(), strpos($head, '</style>'), 0);
				$head = $h;	
			}
			// QR code
			$qrCode = new QrCode();
			/*if (strpos($template,'{qr_code}') !== false) {	
				$qrCode
   	 				->setText("Life is too short to be generating QR codes")
					->setSize(300)
				    ->setPadding(10)
				    ->setErrorCorrection('high')
				    ->setForegroundColor(array('r' => 0, 'g' => 0, 'b' => 0, 'a' => 0))
				    ->setBackgroundColor(array('r' => 255, 'g' => 255, 'b' => 255, 'a' => 0))
				    ->setLabel('My label')
				    ->setLabelFontSize(16)
			}*/

			$event = $this->database->query('
				SELECT events_name, events_date, events_time, kinds_name, address_city, address_street, address_house_num
				FROM events
				JOIN kinds ON id_kinds = events_kind_id
				JOIN address ON id_address = events_address_id
				WHERE id_events = ?
			 ', $event_id)->fetch();

			$seats = json_decode($jsonStr, TRUE);

			foreach($seats as $seat) {
				//Tickets code
				$ticket = $this->database->table(self::TICKETS_TABLE_NAME)->where(array(
					self::TICKETS_EVENTS_ID => $event_id,
					self::TICKETS_SEATS_ID => $seat['seats_id']
				))->select(self::TICKETS_CODE)->fetch();	

				$qr_style = "#qrcode".$ticket->tickets_code."{background: url(".$qrCode->setText($ticket->tickets_code)->getDataUri().") no-repeat; width:148px; height:148px; margin:0px auto;}";
				$h = substr_replace($head, $qr_style, strpos($head, '</style>'), 0);
				$head = $h;
				
				$replace = array($event->kinds_name, $event->events_name, date('M j, Y', strtotime($event->events_date)), $event->events_time->format('%H:%i'), $seat['seats_name'], $barcode->code39($ticket->tickets_code), '<div class="qrcode"><div id="qrcode'.$ticket->tickets_code.'"></div></div>', $ticket->tickets_code, $seat['seats_price'], $event->address_city, $event->address_street, $event->address_house_num);
				array_push($filled, str_replace($search, $replace, $body));			  											
			}			
					
			$html['head'] = $head;
			$html['foot'] = "</body></html>";
			$html['ticket'] = $filled;
			
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
		
		return $html;
	 }
	 

	 /*
	  * Creates VIP seats.
	  * @param string
	  * @param int
	  */
	 public function createVip($jsonStr, $id)
	 {
	 	$json = json_decode($jsonStr, TRUE);	

	 	try {
	 		foreach($json as $seats_id) {
				$this->database->table(self::SEATS_LAYOUT_EVENTS_TABLE_NAME)->where(self::SEATS_LAYOUT_EVENTS_SEATS_ID, $seats_id)->update(array(
					self::SEATS_LAYOUT_EVENTS_AVAILABLE => "VIP_1"
				));	
			}
	 	} catch (\Exception $e) {
	 		throw new \Exception($e);	
	 	}
	 }


	 public function getKind($id)
	 {
	 	return $this->database->table(self::KINDS_TABLE_NAME)->where(self::KINDS_ID, $id)->fetch();
	 }


}