<?php

class MyAuthorizator implements Nette\Security\IAuthorizator
{
    /**
     * @var Nette\Security\Permission
     */

    /** 
     * @var Nette\Database\Context
     */
     
    private $database;

    private $acl;

    public function __construct(Nette\Database\Context $database)
    {
        $this->database = $database;
        $this->acl = new Nette\Security\Permission();

        $roles = $this->database->table('roles')->select('roles_name')->fetchAll();
        $permissions = $this->database->table('permissions')->select('permissions_name')->fetchAll();

        foreach ($permissions as $key => $permission) {
            $this->acl->addResource($permission->permissions_name);
        }

        foreach ($roles as $key => $role) {
            $this->acl->addRole($role->roles_name);
            $this->acl->allow($role->roles_name, $this->getPermissions($role->roles_name));
        }
    }


    function isAllowed($role, $resource, $privilege)
    {
        return $this->acl->isAllowed($role, $resource, $privilege);
    }



    /**
     * Finds permissions.
     * @param string
     * @return array
     */
    public function getPermissions($role)
    {           
        $permissions = $this->database->query("
                SELECT permissions.id_permissions, permissions.permissions_name  
                FROM roles LEFT JOIN permissions ON roles.roles_permissions & permissions.id_permissions
                WHERE roles.roles_name = ?
            ", $role)->fetchPairs();

        return $permissions;
    }


}

?>