<?php

namespace App\Presenters;

use Nette,
	Nextras,
	App\Forms\PermissionsFormFactory,
	App\Forms\ReportFormFactory,
	App\Forms\CurrencyFormFactory,
	App\Model\ConfigManager;


/**
 * Config presenter.
 */
class ConfigPresenter extends BasePresenter
{
	/** @var ConfigManager @inject */
	public $configManager;
	
	/**  @var PermissionsFormFactory @inject */
	public $permissionsForm;

	/**  @var ReportFormFactory @inject */
	public $reportForm;

	/**  @var CurrencyFormFactory @inject */
	public $currencyForm;
	
	
	public function beforeRender()
	{
		parent::beforeRender();
	}


	/**
	 * Permissions DataGrid
	 */
	 public function createComponentPermissionsDataGrid()
	 {
		$grid = new Nextras\Datagrid\Datagrid;
		$grid->addColumn('roles_name', 'Role')->enableSort($grid::ORDER_DESC);
		
		$grid->setFilterFormFactory(function() {
			$form = new Nette\Forms\Container;
			$form->addText('roles_name');
			
			return $form;
		});
		
		$grid->setDataSourceCallBack(function($filter, $order) {
			$filters = array();
		    foreach ($filter as $k => $v) {
		        $filters[$k. ' LIKE ?'] = "%$v%";
		    }
			
		    $selection = $this->database->table('roles')->where('roles_is_active', 1)->where($filters)->select('id_roles, roles_name');

		    return $selection;
		});
		
		
		$grid->addCellsTemplate(__DIR__ .'/templates/Config/@cells.latte');
		
		return $grid;
	 }



	/**
	 * Form for updating permissions.
	 * @return Nette\Application\UI\Form
	 */
	protected function createComponentPermissionsForm()
	{		
		$form = $this->permissionsForm->create();
		
		$form->onSuccess[] = function ($form) {
			if(!empty($form->getValues()->id)){
				$this->flashMessage('The permissions was updated successfully.');
			} else {
				$this->flashMessage('The permissions was created successfully.');
			}
			$this->redirect("Config:");
		};
		
		return $form;
	}


	/**
	 * Deletes role.
	 */
	 public function handleDelete($id){
	 	$this->configManager->delete($id);
	 }


	/**
	 * Updates role.
	 */
	 public function handleEdit($id){
		$this->redirect('Config:create', array('edit' => $id));
	 }


	/**
	 * Form for creating reports.
	 * @return Nette\Application\UI\Form
	 */
	protected function createComponentReportForm()
	{		
		$form = $this->reportForm->create();
		
		return $form;
	}

	

	/*
	 * Handles downloading reports.
	 */
	public function actionDownload($type, $from = "", $to = "")
	{
		switch ($type) {
			case 'last_month':
				$date = date("n-Y", strtotime("first day of previous month"));
				break;
			
			case 'current_month':
				$date = "from_".date("j-n-Y", strtotime("first day of this month"))."_to_".date("j-n-Y", strtotime("today"));
				break;

			case 'date_range':
				$date = "from_".$from."_to_".$to;
				break;
		}

		$report_data = $this->configManager->report($type, $from, $to);

		$response = new \Nette\Application\Responses\CsvResponse($report_data, "report_".$date.".csv");
        $this->sendResponse( $response );
	}


	/**
	 * Form for editing currency.
	 * @return Nette\Application\UI\Form
	 */
	protected function createComponentCurrencyForm()
	{		
		$form = $this->currencyForm->create();
		
		return $form;
	}


}
