<?php

namespace App\Presenters;

use Nette,
	Nextras,
	App\Forms\DealerFormFactory,
	App\Model\DealerManager;


/**
 * Dealer presenter.
 */
class DealerPresenter extends BasePresenter
{
	/** @var DealerManager @inject */
	public $dealerManager;
	
	/**  @var DealerFormFactory @inject */
	public $dealerForm;
	
	
	public function beforeRender()
	{
		parent::beforeRender();
	}


	/**
	 * Dealers DataGrid
	 */
	 public function createComponentDealerDataGrid()
	 {
		$grid = new Nextras\Datagrid\Datagrid;
		$grid->addColumn('dealers_name', 'Name')->enableSort($grid::ORDER_DESC);
		$grid->addColumn('dealers_credit', 'Credit')->enableSort();
		$grid->addColumn('dealers_email', 'E-mail')->enableSort();
		
		$grid->setFilterFormFactory(function() {
			$form = new Nette\Forms\Container;
			$form->addText('dealers_name');
			$form->addText('dealers_email');
			
			return $form;
		});
		
		$grid->setDataSourceCallBack(function($filter, $order) {
			$filters = array();
		    foreach ($filter as $k => $v) {
		        if ($k == 'id' || is_array($v))
		            $filters[$k] = $v;
		        else
		            $filters[$k. ' LIKE ?'] = "%$v%";
		    }
			
		    $selection = $this->database->table('dealers')->where('dealers_is_removed', '0')->where($filters)->select('id_dealers, dealers_name, dealers_email, dealers_credit, dealers_is_active');

		    return $selection;
		});
		
		
		$grid->addCellsTemplate(__DIR__ .'/templates/Dealer/@cells.latte');
		
		return $grid;
	 }


	/**
	 * Deletes dealer.
	 */
	 public function handleDelete($id){
	 	$this->dealerManager->delete($id);
	 }


	/**
	 * Updates dealer.
	 */
	 public function handleEdit($id){
		$this->redirect('Dealer:create', array('edit' => $id));
	 }
	 
	 
	/**
	 * Adds credit to dealer.
	 */
	 public function handleAdd($id, $value){
		$this->dealerManager->addCredit($id, $value);
	 }
	 
	 
	/**
	 * Deducts credit to dealer.
	 */
	 public function handleDeduct($id, $value){
		$this->dealerManager->deductCredit($id, $value);
	 }
	 
	 
	/**
	 * Adds form for creating/editing dealer.
	 * @return Nette\Application\UI\Form
	 */
	protected function createComponentDealerForm()
	{		
		$form = $this->dealerForm->create();
		
		$form->onSuccess[] = function ($form) {
			if(!empty($form->getValues()->id)){
			 	$this->flashMessage('The dealer was updated successfully.');
			} else {
				$this->flashMessage('The dealer was created successfully.');
			}
			$this->redirect('Dealer:');
		};
		
		return $form;
	}
	
	
}
