<?php

namespace App\Presenters;

use Nette,
	Nextras,
	App\Model,
	Nette\Security\User;


/**
 * Homepage presenter.
 */
class HomepagePresenter extends BasePresenter
{
	public function beforeRender()
	{
		parent::beforeRender();
	}


	/**
	 * Actual Events DataGrid
	 */
	public function createComponentActualEventsDataGrid()
	{
		$grid = new Nextras\Datagrid\Datagrid;
		$grid->addColumn('events_name', 'Event')->enableSort();
		$grid->addColumn('events_datetime', 'Date & Time')->enableSort($grid::ORDER_ASC);
		$grid->addColumn('events_available', 'Available')->enableSort();
		$grid->addColumn('events_vip', 'VIP')->enableSort();
		$grid->addColumn('events_sold', 'Sold')->enableSort();
		
		$grid->setDataSourceCallBack(function($filter, $order) {
			
		    $selection = $this->database->query('
		    	SELECT events.events_name AS events_name, CONCAT(events.events_date, events.events_time) AS events_datetime, SUM(seats_layout_events_available = "1") AS events_available, SUM(seats_layout_events_available = "0") AS events_sold, SUM(seats_layout_events_available = "VIP") AS events_vip  
				FROM seats_layout_events
				JOIN events max ON events_seats_layout_events_max >= id_seats_layout_events
				JOIN events ON events.events_seats_layout_events_min <= id_seats_layout_events
				WHERE events.events_date >= SUBDATE( CURDATE(), INTERVAL 5 DAY) AND events.events_is_active = "1"
				GROUP BY events.id_events;
		    ');

		    return $selection;
		});
		
		
		$grid->addCellsTemplate(__DIR__ .'/templates/Homepage/@actualeventscells.latte');
		
		return $grid;
	}


	/**
	 * Last Chance DataGrid
	 */
	public function createComponentLastChanceDataGrid()
	{
		$grid = new Nextras\Datagrid\Datagrid;
		$grid->addColumn('events_name', 'Event')->enableSort();
		$grid->addColumn('events_datetime', 'Date & Time')->enableSort($grid::ORDER_ASC);
		$grid->addColumn('events_available', 'Available')->enableSort();
		$grid->addColumn('events_vip', 'VIP')->enableSort();


		$grid->setDataSourceCallBack(function($filter, $order) {
			
		    $selection = $this->database->query('
				SELECT events.events_name, CONCAT(events.events_date, events.events_time) AS events_datetime, SUM(seats_layout_events_available = "1") AS events_available, SUM(seats_layout_events_available = "VIP") AS events_vip  
				FROM seats_layout_events
				JOIN events max ON events_seats_layout_events_max >= id_seats_layout_events
				JOIN events ON events.events_seats_layout_events_min <= id_seats_layout_events
				WHERE events.events_date >= SUBDATE( CURDATE(), INTERVAL 5 DAY)
				GROUP BY events.id_events
				    HAVING ((events_available + events_vip) <= 10 AND (events_available + events_vip) > 0);
		    ');

		    return $selection;
		});
		
		
		$grid->addCellsTemplate(__DIR__ .'/templates/Homepage/@lastchancecells.latte');
		
		return $grid;
	}


	/**
	 * Sold Out DataGrid
	 */
	 public function createComponentSoldOutDataGrid()
	 {
		$grid = new Nextras\Datagrid\Datagrid;
		$grid->addColumn('events_name', 'Event')->enableSort();
		$grid->addColumn('events_datetime', 'Date & Time')->enableSort($grid::ORDER_ASC);
		
		$grid->setDataSourceCallBack(function($filter, $order) {
			
		    $selection = $this->database->query('
		    	SELECT events.events_name, CONCAT(events.events_date, events.events_time) AS events_datetime  
				FROM seats_layout_events
				JOIN events max ON events_seats_layout_events_max >= id_seats_layout_events
				JOIN events ON events.events_seats_layout_events_min <= id_seats_layout_events
				WHERE events.events_date >= CURDATE()
				GROUP BY events.events_name
				    HAVING SUM(seats_layout_events_available = "1") + SUM(seats_layout_events_available = "VIP") = 0;
		    ');

		    return $selection;
		});
		
		
		$grid->addCellsTemplate(__DIR__ .'/templates/Homepage/@soldoutcells.latte');
		
		return $grid;
	 }

	
}
